# -*- encoding: utf-8 -*-
#素材
require 'spec_helper'

describe ResourcePicture do
  include ManifestsMacros
  before do
    @model = ResourcePicture
  end
  
  describe 'ペタナイズに於いて' do
    it '絵師が投稿するコンテンツである' do
      expect(@model.ancestors.include?(Peta::Content)).to eq true
    end
    it '親モデルは持たない' do
      expect(@model.my_peta.parent_model_name).to be_blank
    end
    describe 'ブーストに於いて' do
      before do
        @boosts = @model.my_peta.boost
        @boosts_names = ['license_group', 'license', 'credit_picture']
      end
      it 'これらのブーストが定義されている' do
        set_boosts? @boosts, @boosts_names
      end
      describe 'ライセンスグループに於いて' do
        before do
          @mani = @boosts['license_group']
        end
        it 'フキダシテンプレート' do
          expect(@mani.level).to eq 'post'
          expect(@mani.template_name).to eq 'license_group'
          expect(@mani.column_name_for_template_module).to eq 'license_group_module_name'
          expect(@mani.settings_column_name).to eq 'license_group_settings'
          expect(@mani.foreign_key).to eq 'license_group_id'
        end
        it 'フキダシテンプレート' do
          expect(@mani.item_name_for_extend_model).to eq 'license'
          expect(@mani.setter_method_name).to eq 'license_extend='
          expect(@mani.getter_method_name).to eq 'license_extend'
          expect(@mani.extend_model_class_name).to eq 'License'
          expect(@mani.extend_module_name).to eq 'LicenseModule'
        end
      end
      describe 'ライセンスに於いて' do
        before do
          @mani = @boosts['license']
        end
        it 'フキダシテンプレート' do
          expect(@mani.level).to eq 'read'
          expect(@mani.template_name).to eq 'license_group'
          expect(@mani.column_name_for_template_module).to eq 'license_group_module_name'
          expect(@mani.settings_column_name).to eq 'license_settings'
          expect(@mani.foreign_key).to eq 'license_group_id'
        end
        it 'フキダシテンプレート' do
          expect(@mani.item_name_for_extend_model).to eq 'attribute'
          expect(@mani.setter_method_name).to eq 'attribute_extend='
          expect(@mani.getter_method_name).to eq 'attribute_extend'
          expect(@mani.extend_model_class_name).to eq 'Attribute'
          expect(@mani.extend_module_name).to eq 'AttributeModule'
        end
      end
      describe 'クレジットに於いて' do
        before do
          @mani = @boosts['credit_picture']
        end
        it 'フキダシテンプレート' do
          expect(@mani.level).to eq 'read'
          expect(@mani.template_name).to eq 'license_group'
          expect(@mani.column_name_for_template_module).to eq 'license_group_module_name'
          expect(@mani.settings_column_name).to eq 'credit_picture_settings'
          expect(@mani.foreign_key).to eq 'license_group_id'
        end
        it 'フキダシテンプレート' do
          expect(@mani.item_name_for_extend_model).to eq 'credit_picture'
          expect(@mani.setter_method_name).to eq 'credit_picture_extend='
          expect(@mani.getter_method_name).to eq 'credit_picture_extend'
          expect(@mani.extend_model_class_name).to eq 'CreditPicture'
          expect(@mani.extend_module_name).to eq 'CreditPictureModule'
        end
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it '実素材のためのコントローラである' do
      expect(@model.item_name).to eq 'resource_picture'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'by_license_group', 'by_license', 'by_artist', 'show', 
          'count', 'count_by_license_group', 'count_by_license', 'count_by_artist']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # 素材の公開リストを返すように定義している
          set_list_action? @action, 'resource_picture', 'public'
        end
      end
      describe 'by_license_groupに於いて' do
        before do
          @action = @actions['by_license_group']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # 素材のライセンスグループフィルタリストを返すように定義している
          set_list_action? @action, 'resource_picture', 'by_license_group'
        end
      end
      describe 'by_licenseに於いて' do
        before do
          @action = @actions['by_license']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # 実素材のライセンスフィルタリストを返すように定義している
          set_list_action? @action, 'resource_picture', 'by_license'
        end
      end
      describe 'by_artistに於いて' do
        before do
          @action = @actions['by_artist']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # 素材の絵師フィルタリストを返すように定義している
          set_list_action? @action, 'resource_picture', 'by_artist'
        end
      end
      describe 'showに於いて' do
        before do
          @action = @actions['show']
        end
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # 素材の公開リストのカウントを返すように定義している
          set_list_action? @action, 'resource_picture', 'public'
        end
      end
      describe 'count_by_license_groupに於いて' do
        before do
          @action = @actions['count_by_license_group']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # 素材のライセンスグループフィルタリストのカウントを返すように定義している
          set_list_action? @action, 'resource_picture', 'by_license_group'
        end
      end
      describe 'count_by_licenseに於いて' do
        before do
          @action = @actions['count_by_license']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # 素材のライセンスフィルタリストのカウントを返すように定義している
          set_list_action? @action, 'resource_picture', 'by_license'
        end
      end
      describe 'count_by_artistに於いて' do
        before do
          @action = @actions['count_by_artist']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # 素材の絵師フィルタリストのカウントを返すように定義している
          set_list_action? @action, 'resource_picture', 'by_artist'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'resource_picture'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'resource_pictures'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = ['original_picture', 'license_group', 'license', 'artist', 'system_picture']
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
        describe 'original_pictureに於いて' do
          before do
            @mani = @belongs_to['original_picture']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'original_picture'
            expect(@mani.id_column).to eq 'original_picture_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq OriginalPicture
          end
        end
        describe 'license_groupに於いて' do
          before do
            @mani = @belongs_to['license_group']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'license_group'
            expect(@mani.id_column).to eq 'license_group_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq LicenseGroup
          end
        end
        describe 'artistに於いて' do
          before do
            @mani = @belongs_to['artist']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'artist'
            expect(@mani.id_column).to eq 'artist_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq Artist
          end
        end
        describe 'licenseに於いて' do
          before do
            @mani = @belongs_to['license']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'license'
            expect(@mani.id_column).to eq 'license_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq License
          end
        end
        describe 'system_pictureに於いて' do
          before do
            @mani = @belongs_to['system_picture']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'system_picture'
            expect(@mani.id_column).to eq 'system_picture_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq SystemPicture
          end
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = []
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'original_picture_id', 'ext', 
          'width', 'height', 'filesize', 'artist_id', 'license_group_id', 'license_id', 
          'md5', 'system_picture_id', 
          'license_group_settings', 'credit_picture_settings', 
          'license_settings', 'picture_id', 'license_group_module_name', 
          'created_at', 'updated_at']
      end
      describe 'original_picture_idに於いて' do
        before do
          @attribute = @attributes['original_picture_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'extに於いて' do
        before do
          @attribute = @attributes['ext']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'widthに於いて' do
        before do
          @attribute = @attributes['width']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'heightに於いて' do
        before do
          @attribute = @attributes['height']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'filesizeに於いて' do
        before do
          @attribute = @attributes['filesize']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'artist_idに於いて' do
        before do
          @attribute = @attributes['artist_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'license_group_idに於いて' do
        before do
          @attribute = @attributes['license_group_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'license_idに於いて' do
        before do
          @attribute = @attributes['license_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'md5に於いて' do
        before do
          @attribute = @attributes['md5']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'system_picture_idに於いて' do
        before do
          @attribute = @attributes['system_picture_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'license_group_settingsに於いて' do
        before do
          @attribute = @attributes['license_group_settings']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'credit_picture_settingsに於いて' do
        before do
          @attribute = @attributes['credit_picture_settings']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'license_settingsに於いて' do
        before do
          @attribute = @attributes['license_settings']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'picture_idに於いて' do
        before do
          @attribute = @attributes['picture_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'license_group_module_nameに於いて' do
        before do
          @attribute = @attributes['license_group_module_name']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
    end
    
  end
  
end
